
import java.sql.*;
import java.util.*;
import java.net.*;
import java.io.*;

import org.apache.regexp.RE;


public class ClassifiedReader {
	
	Connection con = null;
	
	private String CARS_COM_URL = "http://www.cars.com/carsapp/national/?szc=%zip%&srv=adlocator&act=search&pn=%page%&ps=30&rd=%miles%&mknm=%make%&mdnm=All&vp=All&yr=%281930%3A2005%29&flt=n_ms,zr,used&sb=&so=DESC&_html=t&x=14&y=7";
	private String CARS_COM_MILES = "30";
	private String CARS_COM_ZIP = "95722";

	private String AUBURN_JOURNAL_URL = "http://www.placernews.com/classifieds/index.inn?loc=detail&main=Transportation";

	
	private boolean openDatabase() {
		
		//load driver
		try {
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
			return false;
		}
		
				
		try {
			String url = "jdbc:odbc:Sierra";
			con = DriverManager.getConnection(url, "", "");
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");
			System.err.println(ex.getMessage());
			return false;
		}
		
		return true;
		
	}
	
	public void closeDatabase() {
		try {
			con.close();
		} catch(SQLException ex) {
			System.err.print("SQLException: ");
			System.err.println(ex.getMessage());
		}
	}
		
		

	private boolean isTableReady() {
	
		String query = "SELECT COUNT(*) FROM New_Classified_Ads";
		int recordCount = 0;
		
		try {
			
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			
			rs.next();
			recordCount = rs.getInt(1);
			stmt.close();
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");		
			System.err.println(ex.getMessage());			
			return false;
		}
		
		if(recordCount>0) {
			System.out.println("Data must be transferred prior to processing new records.");
			return false;
		}
		else
			return true;
	
	}
	
	private Vector getCarsComMakes() {

		String query = "SELECT * FROM Content_Cars_Com";
		Vector makes = new Vector();
	
		try {
			
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			
			while(rs.next()) {
			
				String make = rs.getString(1);
				makes.add(make);
			}
			
			stmt.close();
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");		
			System.err.println(ex.getMessage());						
		}
		
		return makes;
			
					
	}
	
	
	private String getHtml(String addr) {
	     
	     	StringBuffer buffer = new StringBuffer();
	     	URL url = null;
	     	DataInputStream in = null;
	     	
		try { 
	       		url = new URL(addr);
        	} catch(MalformedURLException e) {
        		System.err.println("MalformedURLException: ");
        		System.err.println(e.getMessage());
        	}

        	try {
            		in = new DataInputStream(url.openStream());

			int i;
			char ch;
	    		while ((i=in.read()) != -1) {
	    			ch=(char)i;
				buffer.append(ch);
			}	    		
	    
		} catch(Exception e) {
			System.err.println(e.toString());
			
		}
		
		try {
			in.close();
		} catch(Exception e) {}
		
	
		return buffer.toString();
	}
	
	
	
	private void processCarsCom() {
		System.out.println("Processing cars.com");
		
		Vector makes = getCarsComMakes();
		
		for(Enumeration enum = makes.elements(); enum.hasMoreElements();) {
			String make = (String)enum.nextElement();
			System.out.println(" --- processing " + make);			
			
			int page = 0;
			boolean more = true;
			
			while(more) {
			
				page++;
				more = false;
				System.out.println(" ----- processing page " + page);
				
				String addr = CARS_COM_URL;
		            	
		            	addr = substitute(addr, "%zip%", CARS_COM_ZIP);		            	
		            	addr = substitute(addr, "%miles%", CARS_COM_MILES);
		            	addr = substitute(addr, "%make%", make);
		            	
		            	Integer pageInt = new Integer(page);		            	
		            	addr = substitute(addr, "%page%", pageInt.toString());
		            		            	
		            	String html = getHtml(addr);		            
				more = parseCarsCom(html);
		
			}

		}
	
	}
	
	private String substitute(String url, String subVal, String insVal) {
	
		String outUrl = null;	
		int i;
      
   		i = url.indexOf(subVal);
   		
   		if(i>0) {   		
   			outUrl = url.substring(0,i) + insVal + url.substring(i+subVal.length());
		}   

   		return outUrl;
   	
	}
	
	
	private boolean parseCarsCom(String html) {


		String NO_MATCH_STR = "";	
	 	String START_SECTION = "<!-- START OF TI TAG FOR RESULTS SET!! -->";
   		String END_SECTION = "<!-- END OF TI TAG!! -->";
   		String START_REC = "<TABLE";
   		String END_REC = "</TABLE>";
   		String MORE_REC = "Next &gt;&gt;";
   
   		char quote = 34;
   
   		String START_YEAR = quote + ">";
   		String START_MAKE = "pn=";
		String END_MAKE = "</";   		
		String START_PRICE = quote + ">";   
		String END_PRICE = "</";   
		String START_MILES = quote + ">";   
		String END_MILES = "</";   
		String START_SELLER = quote + ">";
		String END_SELLER = "</TD>";
		String START_PHONE = quote + ">";
		String END_PHONE = "</";   
   
   		int i, j;
      		String yr = null, make = null, price = null, miles = null, seller = null, phone = null, desc = null;
      		
      		String startHtml = html;

		//check for no matches
		if(!(html.indexOf(NO_MATCH_STR)>0)) {
   
   		
   			//find first entry
   			if(html.indexOf(START_SECTION)>0) {         

				i = html.indexOf(START_SECTION);
				html = html.substring(i);
				      
				if(html.indexOf(END_SECTION)>0) {
					
					i = html.indexOf(END_SECTION);
					html=html.substring(0,i);
					         
         				//System.out.println("\n"+html.substring(0,300)+"\n");
            
    
 					//parse table
			            	i = html.indexOf(START_REC);
			            
			            	while(i > 0) {
			             
			            		html = html.substring(i);
			               		//System.out.println("\n" +html.substring(0,300) + "\n");
			               		
			               		//yr
			             		if(html.indexOf(START_YEAR)>0) {
			             			i = html.indexOf(START_YEAR);
			             			html = html.substring(i+START_YEAR.length());
			             			//System.out.println(html.substring(0,300));
			             			
			             			yr = html.substring(0,4);
			             			//System.out.println("yr: " + yr);
			             		}
			             		
			             		//make
						if(html.indexOf(START_MAKE)>0) {
							i = html.indexOf(START_MAKE);
							html = html.substring(i+START_MAKE.length());
							i = html.indexOf(">");
							html = html.substring(i+1);
							j = html.indexOf(END_MAKE);
							make = html.substring(0,j);
							//System.out.println("make: " + make);
						}
						
						//price
						if(html.indexOf(START_PRICE)>0) {
							i = html.indexOf(START_PRICE);
							html = html.substring(i+START_PRICE.length());
							j = html.indexOf(END_PRICE);
							price = html.substring(0,j);
							//System.out.println("price: " + price);
						}
						
						//miles
						if(html.indexOf(START_MILES)>0) {
							i = html.indexOf(START_MILES);
							html = html.substring(i+START_MILES.length());
							j = html.indexOf(END_MILES);
							miles = html.substring(0,j);
							
						}
						
						if(miles.equals("<BR>"))
							miles = "";
							
						//System.out.println("miles: " + miles);

						if(html.indexOf(START_SELLER)>0) {
							i = html.indexOf(START_SELLER);
							html = html.substring(i+START_SELLER.length());
							j = html.indexOf(END_SELLER);
							seller = html.substring(0,j);
							html = html.substring(j);
							//System.out.println("seller: " + seller);
						}
						
						if(html.indexOf(START_PHONE)>0) {
							i = html.indexOf(START_PHONE);
							html = html.substring(i+START_PHONE.length());
							j = html.indexOf(END_PHONE);
							phone = html.substring(0,j);
							//System.out.println("phone: " + phone);
						}
						
			               
				               //find end of record
				               i = html.indexOf(END_REC);
				               html = html.substring(i + END_REC.length());
				                     			               
				               //find beginning again
				               i = html.indexOf(START_REC);
				               
			               
						if(seller.equals("") || seller.equals("Exclusive Online Classified")) {
				               			            
				                  	desc = yr + " " + make + ";" + price + ";" + miles + ";" + phone;
				                  
				                  	if(seller.equals("Exclusive Online Classified")) {
				                  		desc = desc + ";Online";
				                  	}
				                
				                  	addEntry(desc,phone,"Cars.com");
				                }
				           				      
					}
				
					if(startHtml.indexOf(MORE_REC)>0) {
						return true;
					}
					else {
						return false;
					}	
					      				
				}
					
			}
			
		}
	
		
		return false;	
	}
	
	private boolean inCallingArea(String phone) {

		String areaCode = phone.substring(0,3);
		String prefix = phone.substring(4,7);		
		String query = "SELECT count(*) FROM CallingArea WHERE Prefix = '" + prefix + "' and AreaCode = '" + areaCode + "'";
		
		int recordCount = 0;		   
		
		try {
			
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			
			rs.next();
			recordCount = rs.getInt(1);
			stmt.close();
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");		
			System.err.println(ex.getMessage());			
			return false;
		}
		
		if(recordCount>0) {
			System.out.println("In calling area.");
			return true;
		}		
		else {
			System.out.println("Out of calling area.");
			return false;
		}
				
	}
	
	private boolean isCalled(String phone) {


		String query ="SELECT count(*) FROM Called WHERE PhoneNo ='" + phone + "'";
		
		int recordCount = 0;		   
		
		try {
			
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			
			rs.next();
			recordCount = rs.getInt(1);
			stmt.close();
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");		
			System.err.println(ex.getMessage());			
			return false;
		}
		
		if(recordCount>0) {
			System.out.println("Previously called.");
			return true;
		}		
		else {
			System.out.println("Not called.");
			return false;
		}	
	}
	
	private boolean isInList(String phone) {
		

		String query ="SELECT count(*) FROM New_Classified_Ads WHERE PhoneNo ='" + phone + "'";
		
		int recordCount = 0;		   
		
		try {
			
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			
			rs.next();
			recordCount = rs.getInt(1);
			stmt.close();
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");		
			System.err.println(ex.getMessage());			
			return false;
		}
		
		if(recordCount>0) {
			System.out.println("Already in list.");
			return true;
		}		
		else {
			System.out.println("Not in list.");
			return false;
		}	
		
	}
	
	private String removeQuotes(String str) {
		
		StringBuffer buffer = new StringBuffer();
		char temp = '\'';
		Character ch;
		Character quote= new Character(temp);
		
		for(int i = 0; i < str.length(); i++) {
			ch = new Character(str.charAt(i));
			if(!ch.equals(quote))
				buffer.append(ch);
		}
		
		return buffer.toString();

		
	}
		
	private void addEntry(String desc, String phone, String source) {
	
		System.out.println("\ndesc: " + desc);
		System.out.println("phone: " + phone);
		System.out.println("source: " + source);
		
		String prefix = null;
		StringBuffer sql = null;
		   
   		phone = phone.trim();
   		if(!phone.equals("")) {
   			    
		      //check area code (916, 530)
		      String areaCode = phone.substring(0,3);
		      System.out.println(areaCode);
		      
		      if(areaCode.equals("916") || areaCode.equals("530")) {
		      	      			
      				//check prefix
      				prefix = phone.substring(4,7);
      				System.out.println(prefix);
      				
				if(inCallingArea(phone)) {

					if(!isCalled(phone) && !isInList(phone)) {
						            
						desc = removeQuotes(desc);               
						
						sql = new StringBuffer();
						sql.append("INSERT INTO New_Classified_Ads (PhoneNo, AreaCode, Prefix, Description, Source) VALUES (");
               					sql.append("'").append(phone).append("'");
               					sql.append(",'").append(areaCode).append("'");
               					sql.append(",'").append(prefix).append("'");
               					sql.append(",'").append(desc).append("'");
               					sql.append(",'").append(source).append("')");
						
						//System.out.println(sql);               
               
						//insert
						try {			
							Statement stmt = con.createStatement();
							stmt.executeUpdate(sql.toString());			
							stmt.close();
			
						} catch(SQLException ex) {
							System.err.print("SQLException: ");		
							System.err.println(ex.getMessage());			
						}																			
					}
				}
			}
		}		
	}
	
	private void processSacBee() {
		System.out.println("Processing Sac Bee");

		Vector sections = getSacBeeSections();
		
		for(Enumeration enum = sections.elements(); enum.hasMoreElements();) {
			String section = (String)enum.nextElement();
			System.out.println(" --- processing section: " + section);			
		            		            	
	            	String html = getHtml(section);		            
			parseSacBee(html);		
		}		
		
		
		
		
	}
	
	private void parseSacBee(String html) {
	
	
	
		String START_REC = "classads/dingbats/listings_bug.gif";
		String END_REC = "<img";
		
		int i, j;
		String desc = null;
		String data = null;
		String rec = null;
		String phoneNo = null;
		
		data = html;
		
		//find first entry
		try {
			
			if(data.indexOf(START_REC)>-1) {
			
				i = data.indexOf(START_REC);
			
				while(data.length()>0) {
				
					data = data.substring(i+54);
				
					if(data.indexOf(END_REC)>-1) {
					
						i = data.indexOf(END_REC);
						rec = data.substring(0,i);
				
						rec = rec.trim();
					
						if(!rec.equals("")) {
									
               						//System.out.println(rec);
               						phoneNo = findPhoneNo(rec, "916");
               						addEntry(rec, phoneNo, "Sacramento Bee");
               					
               					}
                     
						i = data.indexOf(END_REC);
						if(i>-1) {
							data = data.substring(i);
						}
						else
							data = "";
					
						i = data.indexOf(START_REC);
					
					}
				
				}
			}	
		}
		catch(Exception e) {}
		
	}
	

	private Vector getSacBeeSections() {

		String query = "SELECT * FROM Content_SacBee";
		Vector sections = new Vector();
	
		try {
			
			Statement stmt = con.createStatement();
			ResultSet rs = stmt.executeQuery(query);
			
			while(rs.next()) {
			
				String section = rs.getString("URL");
				sections.add(section);
			}
			
			stmt.close();
			
		} catch(SQLException ex) {
			System.err.print("SQLException: ");		
			System.err.println(ex.getMessage());						
		}
		
		return sections;
			
					
	}
		
	private void processAuburnJournal() {
		System.out.println("Processing Auburn Journal");
	
		String START_SECTION = "</TD></TR></TABLE><HR>";
		String END_SECTION = "<TABLE";
		String START_REC = "<HR>";
		String END_REC = "<HR>";
		
		int i, j;
		String desc = null, rec = null, html = null, phoneNo = null;

		html = getHtml(AUBURN_JOURNAL_URL);

		if(html.indexOf(START_SECTION)>0) {
			
			i = html.indexOf(START_SECTION);
			html = html.substring(i);
			
			if(html.indexOf(END_SECTION)>0) {
				i = html.indexOf(END_SECTION);
				html = html.substring(0,i);

				//parse table
											
				i = html.indexOf(START_REC);
				while(i>0) {
					
            				html = html.substring(i+START_REC.length());
            				i = html.indexOf(END_REC);
            				if(i>0) {
            					rec = html.substring(0,i);
            					rec = rec.trim();
            					
            					//System.out.println(rec);
            					
            					if(!(rec.equals(""))) {

							
							String test = rec.substring(0,2);
														
							if(test.equals("<A")) {
								i = rec.indexOf("<BR>");
								if(i>0) {
									rec = rec.substring(i+4);
								}
							}
							
							rec = rec.trim();
							//System.out.println(rec);
							/*
                  					//remove control characters
                  					boolean keepGoing = true;
                  					int k = 0;
							while(keepGoing) {																
								if(!(Character.isLetterOrDigit(rec.charAt(k)) || Character.isSpaceChar(rec.charAt(k)))) {
									keepGoing = false;
								}
								k++;
							}
							rec = rec.substring(0,k);
							System.out.println(rec);
							*/
								
							phoneNo = findPhoneNo(rec,"530");
							//System.out.println(phoneNo);
                     					
                     					if(phoneNo!=null) {
								addEntry(rec,phoneNo,"Auburn Journal");
								phoneNo = null;
							}
						}  //not blank record
						
					} //if end rec is found
					
					//System.out.println(html);
		                     	//i = html.indexOf(END_REC);
		                     	//if(i>=0) {
		                     	//	System.out.println("made it");
		                     	//	html = html.substring(i);
		                	//}
		                	
		                	i = html.indexOf(START_REC);		                	
		                
		
				} //end while
			} //if			
                }  //if
	} //method

	String findExpression(String str, String expression, int expressionLength) {
		
		RE r = null;		
		boolean match = false;
		String holder = null;
		
		try {
			r = new RE(expression);	

			for(int i = 0; (i+expressionLength)<(str.length()+1); i++) {			
				holder = str.substring(i,i+expressionLength);
				//System.out.println(holder);
				match = r.match(holder);
				//System.out.println(match);
				if(match) return holder;
			}
			
		}
		catch(Exception e) {}
		
		return null;
		
	}					
		
	//private
	public String findPhoneNo(String desc, String defaultPrefix) {
		
		String phone = null;
		
		//find 916-988-2581
		phone = findExpression(desc,"[0-9]{3}-[0-9]{3}-[0-9]{4}",12);
		
		if(phone!=null) {
			//System.out.println(phone);
			return phone;
		}
				
		//find (530) 823-9025, convert to xxx-xxx-xxxx		
		phone = findExpression(desc,"[(][0-9]{3}[)].[0-9]{3}-[0-9]{4}",14);
		
		if(phone!=null) {
			phone = phone.substring(1,4) + "-" + phone.substring(6,9) + "-" + phone.substring(10,14); 
			//System.out.println(phone);
			return phone;
		}
				
		//find (530)889-1046, convert to xxx-xxx-xxxx
		phone = findExpression(desc,"[(][0-9]{3}[)][0-9]{3}-[0-9]{4}",13);
		
		if(phone!=null) {
			phone = phone.substring(1,4) + "-" + phone.substring(5,8) + "-" + phone.substring(9,13); 
			//System.out.println(phone);
			return phone;
		}
				
		//find XXX-XXXX, add prefix
		phone = findExpression(desc,"[0-9]{3}-[0-9]{4}",8);

		if(phone!=null) {
			phone = defaultPrefix + "-" + phone;
			//System.out.println(phone);
			return phone;
		}
			
		return null;
		
	}
	
	public void process() {
		
		
		if(openDatabase()) {
			if(isTableReady()) {		
				processSacBee();
				processAuburnJournal();	
				processCarsCom();
	
			}
		
			closeDatabase();	
		}

	}

	
	static public void main(String args[]) {
		
		ClassifiedReader classifiedReader = new ClassifiedReader();
		classifiedReader.process();   

	}
	
}
	
	